#!/bin/bash

##### Versions used: bwa/0.7.13; samtools/1.10



##### Create BWA index

bwa index CTV_11_accessions.fasta

##### Run BWA mem for paired end data in separate files

bwa mem -t 7 CTV_11_accessions.fasta Sample_1.fastq Sample_2.fastq > Sample.sam

##### Convert sam to bam

samtools view -@ 7 -S -b Sample.sam > Sample.bam

##### Remove sam file

rm Sample.sam

##### Filter sam file for only mapped reads

samtools view -@ 7 -b -F 4 Sample.bam > Sample_mapped.bam

##### Filter sam file for only mapped reads with a 95% identity over 90% of the read

samtools view -@ 7 -h Sample_mapped.bam\
| awk -v maxpct=0.05 -v fracpct=0.9 -v OFS='\t' -v FS='\t' '{
    if($0 ~ /^@/){
        print $0
    } else {
        xnm=gensub(/.*\tNM:i:/, "", "g", $0); 
        nm=gensub(/\t.*/, "", "g", xnm);
        ln=length($10)
        lnfrac=ln*fracpct
        pct=nm/lnfrac;
        if(pct <= maxpct) {
            print $0, "XP:f:"pct
        }
    }
}' > Sample_mapped_pct.sam

##### Convert new sam file to bam file

samtools view -@ 7 -S -b Sample_mapped_pct.sam > Sample_mapped_pct.bam

##### Sort bam file according to coordinates

samtools sort -@ 7 Sample_mapped_pct.bam > Sample_mapped_pct_sorted.bam

##### Create a text file with a line for each base of the reference that was covered with reads.

samtools depth Sample_mapped_pct_sorted.bam > Sample_mapped_pct_sorted_depth.txt

##### Create a text file with the read count per accession including in the read mapping.

samtools view -@ 7 Sample_mapped_pct.sam | cut -f3 | sort | uniq -c | sed -e 's/^[ \t]*//' | tr ' ' \\t > Sample_mapped_names_reads.txt

##### Create a list of the accession names used in the read mapping.

cut -f2 Sample_mapped_names_reads.txt > Sample_Accession_list.txt

##### Filter the text file with all the bases covered with reads per accession to create one file per accession.

while read line
do

grep -w "$line" Sample_mapped_pct_sorted_depth.txt >> Sample_$line\.txt

done < Sample_Accession_list.txt

##### Create a list of only the read counts per accession (remove accession names).

cut -f1 Sample_mapped_names_reads.txt > Sample_Readcounts.txt

##### Paste together accession name in column 1 and the read count per accession in column 2.

paste Sample_Accession_list.txt Sample_Readcounts.txt >> Sample_Mapping_results.txt

##### Create a text file with reference length (column 1), number of bases covered (column 2) and genome coverage (%) (column 3) for each accession that had reads that mapped to it.

cat Sample_Accession_list.txt | while read accession; do

genome_covered=$(cat Sample_$accession\.txt | wc -l | awk '{print $1}')
genome_length=$(grep -m1 $accession Sample_mapped_pct.sam | cut -d ":" -f3)
genome_cov=$(echo "($genome_covered/$genome_length)*100" | bc -l)

printf $genome_length'\t'$genome_covered'\t'$genome_cov'\n' >> Sample_Mapping_results2.txt

done

##### Paste together the accession name and read count to the file containing the genome coverage data.

paste Sample_Mapping_results.txt Sample_Mapping_results2.txt >> Sample_Mapping_results3.txt


##### All intermediate files can be deleted. Final results are in file "Sample_Mapping_results3.txt".



